//
// $Id: MessagesLookup.java 133 2009-08-25 22:57:20Z mdb $

package com.threerings.gwt.util;

import com.samskivert.text.MessageUtil;

/**
 * Dynamic message translation at run-time.
 */
public abstract class MessagesLookup
{
    public static @interface Lookup
    {
        /** The full class name of the Messages class backing our lookups. */
        String using ();
    }

    /**
     * Translate a compound key/params string. The compoundKey is compatible with Narya's
     * MessageBundle, with the exception that qualified keys are not supported.
     */
    public String xlate (String compoundKey)
    {
        // to be more efficient about creating unnecessary objects, check before splitting
        int tidx = compoundKey.indexOf('|');
        if (tidx == -1) {
            return get(compoundKey);

        } else {
            String key = compoundKey.substring(0, tidx);
            String[] args = MessageUtil.decompose(compoundKey.substring(tidx+1));
            for (int ii = 0; ii < args.length; ii++) {
                if (MessageUtil.isTainted(args[ii])) {
                    args[ii] = MessageUtil.untaint(args[ii]);
                } else {
                    args[ii] = xlate(args[ii]);
                }
            }
            return get(key, (Object[])args);
        }
    }

    /** Translate a key with any number of arguments, backed by the Lookup. */
    public String get (String key, Object... args)
    {
        // First make the key for GWT-friendly
        return fetch(key.replace('.', '_'), args);
    }

    /** This is typically generated by MessagesLookupGenerator. */
    protected abstract String fetch (String key, Object... args);
}
